#include "aa.h"

static const char abc_aas[]   = "ABCDEFGHIKLMNPQRSTVWXYZ";
static const char acdbz_aas[] = "ACDEFGHIKLMNPQRSTVWYBZ";
static const char acdx_aas[]  = "ACDEFGHIKLMNPQRSTVWYX";
static const char acd_aas[]   = "ACDEFGHIKLMNPQRSTVWY";
static const char arn_aas[]   = "ARNDCQEGHILKMFPSTWYV";

const int AA_abc_length = sizeof(abc_aas)/sizeof(char) - 1;
const int AA_acdbz_length = sizeof(acdbz_aas)/sizeof(char) - 1;
const int AA_acdx_length = sizeof(acdx_aas)/sizeof(char) - 1;
const int AA_acd_length = sizeof(acd_aas)/sizeof(char) - 1;
const int AA_arn_length = sizeof(arn_aas)/sizeof(char) - 1;

/*A  B  C  D  E  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z*/
static const int abc_indices[] = 
{ 0, 1, 2, 3, 4, 5, 6, 7, 8,-1, 9,10,11,12,-1,13,14,15,16,17,-1,18,19,20,21,22};
static const int acdbz_indices[] = 
{ 0,20, 1, 2, 3, 4, 5, 6, 7,-1, 8, 9,10,11,-1,12,13,14,15,16,-1,17,18,-1,19,21};
static const int acdx_indices[] = 
{ 0,-1, 1, 2, 3, 4, 5, 6, 7,-1, 8, 9,10,11,-1,12,13,14,15,16,-1,17,18,20,19,-1};
static const int acd_indices[] = 
{ 0,-1, 1, 2, 3, 4, 5, 6, 7,-1, 8, 9,10,11,-1,12,13,14,15,16,-1,17,18,-1,19,-1};
static const int arn_indices[] = 
{ 0,-1, 4, 3, 6,13, 7, 8, 9,-1,11,10,12, 2,-1,14, 5, 2,15,16,-1,19,17,-1,18,-1}; 
/*A  B  C  D  E  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z*/

int
AA_abc_index (char aa) {
  int index = (aa < 'A') ? aa - 'a' : aa - 'A';
  if ((index >= 0) && (index < LETTERS)) {
    return abc_indices[index];
  } else {
    return abc_indices['X'-'A'];
  }
}

int
AA_acdbz_index (char aa) {
  int index = (aa < 'A') ? aa - 'a' : aa - 'A';
  if ((index >= 0) && (index < LETTERS)) {
    return acdbz_indices[index];
  } else {
    return acdbz_indices['X'-'A'];
  }
}

int
AA_acdx_index (char aa) {
  int index = (aa < 'A') ? aa - 'a' : aa - 'A';
  if ((index >= 0) && (index < LETTERS)) {
    return acdx_indices[index];
  } else {
    return acdx_indices['X'-'A'];
  }
}

int
AA_acd_index (char aa) {
  int index = (aa < 'A') ? aa - 'a' : aa - 'A';
  if ((index >= 0) && (index < LETTERS)) {
    return acd_indices[index];
  } else {
    return acd_indices['X'-'A'];
  }
}

int
AA_arn_index (char aa) {
  int index = (aa < 'A') ? aa - 'a' : aa - 'A';
  if ((index >= 0) && (index < LETTERS)) {
    return arn_indices[index];
  } else {
    return arn_indices['X'-'A'];
  }
}

char
AA_abc_aa (int i) {
  return abc_aas[i];
}

char
AA_acdbz_aa (int i) {
  return acdbz_aas[i];
}

char
AA_acdx_aa (int i) {
  return acdx_aas[i];
}

char
AA_acd_aa (int i) {
  return acd_aas[i];
}

char
AA_arn_aa (int i) {
  return arn_aas[i];
}

static const double acdbz_freqs[] = {
  0.0754,			/* A */
  0.0170,			/* C */
  0.0529,			/* D */
  0.0631,			/* E */
  0.0406,			/* F */
  0.0686,			/* G */
  0.0223,			/* H */
  0.0572,			/* I */
  0.0594,			/* J */
  0.0931,			/* K */
  0.0236,			/* L */
  0.0454,			/* M */
  0.0491,			/* P */
  0.0402,			/* Q */
  0.0515,			/* R */
  0.0719,			/* S */
  0.0576,			/* T */
  0.0652,			/* V */
  0.0126,			/* W */
  0.0321,			/* Y */
  0.0529+0.0454,		/* B */
  0.0631+0.0402			/* Z */
};

double
AA_acdbz_freq (int i) {
  return acdbz_freqs[i];
}

double
AA_acd_freq (int i) {
  return acdbz_freqs[i];
}

/*
double PartD = acd_freqs[acd_indices['D'-'A']]/acd_freqs[acd_indices['B'-'A']];
double PartN = acd_freqs[acd_indices['N'-'A']]/acd_freqs[acd_indices['B'-'A']];
double PartE = acd_freqs[acd_indices['E'-'A']]/acd_freqs[acd_indices['Z'-'A']];
double PartQ = acd_freqs[acd_indices['Q'-'A']]/acd_freqs[acd_indices['Z'-'A']];
*/
   
