#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define AAS 22
#define LETTERS 26

static char table_aa_comp[AAS] =
"ACDEFGHIKLMNPQRSTVWYBZ";

static int table_index_comp[LETTERS] =
/*A  B  C  D  E  F  G  H  I  J  K  L  M  N  O  P  Q  R  S  T  U  V  W  X  Y  Z*/
{ 0,20, 1, 2, 3, 4, 5, 6, 7,-1, 8, 9,10,11,-1,12,13,14,15,16,-1,17,18,-1,19,21};

extern int
aa_index_comp (char c) {
  return table_index_comp[c - 'A'];
}

extern char
index_aa_comp (int i) {
  return table_aa_comp[i];
}

static double marg[] = {
  0.0754,                       /* A */
  0.0170,                       /* C */
  0.0529,                       /* D */
  0.0631,                       /* E */
  0.0406,                       /* F */
  0.0686,                       /* G */
  0.0223,                       /* H */
  0.0572,                       /* I */
  0.0594,                       /* J */
  0.0931,                       /* K */
  0.0236,                       /* L */
  0.0454,                       /* M */
  0.0491,                       /* P */
  0.0402,                       /* Q */
  0.0515,                       /* R */
  0.0719,                       /* S */
  0.0576,                       /* T */
  0.0652,                       /* V */
  0.0126,                       /* W */
  0.0321,                       /* Y */
  0.0529+0.0454,                /* B */
  0.0631+0.0402                 /* Z */
};

static double
print_freqmatrix (double **matrix1, int n_positions) {
  int j, b;

  for (j = 0; j < n_positions; ++j) {
    for (b = 0; b < 20; ++b) {
      printf("%f ",matrix1[j][b]);
    }
    printf("\n");
  }
}

static double
print_infomatrix (double **matrix, int n_positions) {
  int i, j;

  for (i = 0; i < n_positions; ++i) {
    for (j = 0; j < n_positions; ++j) {
      printf("%f ",matrix[i][j]);
    }
    printf("\n");
  }
}


static double
compute_relent (double *p, double *q) {
  double entropy = 0.0;
  int b;

  for (b = 0; b < 20; ++b) {
    if (p[b] != 0.0 && q[b] != 0.0) {
      entropy += p[b]*log(p[b]/q[b]);
    }
  }
  return entropy*log(2.0);
}

int
main (int argc, char *argv[]) {
  int n_sequences, n_positions, i, j, b, total;
  int **block;
  double **freqmatrix, **infomatrix, total_info, min_info;
  double entropy;
  char c;
  
  fscanf(stdin,"%d %d\n",&n_sequences,&n_positions);
  block = (int **) calloc(n_sequences,sizeof(int *));
  for (i = 0; i < n_sequences; ++i) {
    block[i] = (int *) calloc(n_positions,sizeof(int));
  }

  for (i = 0; i < n_sequences; ++i) {
    for (j = 0; j < n_positions; ++j) {
      fscanf(stdin,"%c",&c);
      if ((b = aa_index_comp(c)) < 20 && b > -1) {
	block[i][j] = b;
      } else {
	block[i][j] = -1;
      }
    }
    fscanf(stdin,"\n");
  }

  freqmatrix = (double **) calloc(n_positions,sizeof(double *));
  for (j = 0; j < n_positions; ++j) {
    freqmatrix[j] = (double *) calloc(20,sizeof(double));
  }

 /* Fill freqmatrix */
  for (j = 0; j < n_positions; ++j) {
    total = 0;
    for (i = 0; i < n_sequences; ++i) {
      if ((b = block[i][j]) >= 0) {
	freqmatrix[j][b] += 1.0;
	++total;
      }
    }
    for (b = 0; b < 20; ++b) {
      freqmatrix[j][b] += marg[b];
    }
    for (b = 0; b < 20; ++b) {
      freqmatrix[j][b] /= ((double) total + 1.0);
    }
  }

  infomatrix = (double **) calloc(n_positions,sizeof(double *));
  for (j = 0; j < n_positions; ++j) {
    infomatrix[j] = (double *) calloc(n_positions,sizeof(double));
  }

  for (i = 0; i < n_positions; ++i) {
    for (j = 0; j < n_positions; ++j) {
      infomatrix[i][j] = compute_relent(freqmatrix[i],freqmatrix[j]);
    }
  }

  /*
  entropy = 0.0;
  for (i = 0; i < n_positions; ++i) {
    entropy += compute_relent(freqmatrix[i],marg);
  }
  */

  total_info = 0.0;
  for (i = 0; i < n_positions; ++i) {
    min_info = 9999999;
    for (j = 0; j < n_positions; ++j) {
      if (j != i) {
	if (infomatrix[i][j] < min_info) {
	  min_info = infomatrix[i][j];
	}
      }
    }
    total_info += min_info/compute_relent(freqmatrix[i],marg);
  }

  /*
  printf("%.3f %.3f\n",
	 total_info,total_info/(double) n_positions);
  */
  printf("%.3f\n",total_info);
  return 0;
}
