/*
 * File:  intlist.h
 * Date:  09-03-99
 * Programmers:  Thomas Wu (thomas.wu@Stanford.Edu)
 *               Craig Nevill-Manning (nevill@cs.rutgers.edu)
 * Comment Writer:  Jimmy Y. Huang (yhuang@leland.stanford.edu)
 * ------------------------------------------------------------
 * This file defines the interface for the Intlist_T ADT.  The Intlist_T
 * ADT is essentially a dynamic int buffer.  The Initlist_T dynamic int
 * buffers are used by emotif-3.0/src/emotif-search/sequence.h to store
 * strings of ASCII integers of various lengths.
 * ------------------------------------------------------------
 * Copyright 1999 The Board of Trustees of 
 * The Leland Stanford Junior University. All Rights Reserved.
 */


#ifndef INTLIST_INCLUDED
#define INTLIST_INCLUDED


/*
 * ----------
 * The statement #define T Intlist_T will replace all instances of the
 * character 'T' with the string "Intlist_T" after this point.
 * ----------
 */

#define T Intlist_T


/*
 * ----------
 * Intlist_T type
 * ==========
 * The Intlist_T ADT is a pointer-to-struct.  The struct the Intlist_T
 * ADT points to will be implemented in the intlist.c file.
 * ----------
 */

typedef struct T *T;


/*
 * ----------
 * Intlist_push
 * ==========
 * This function will allocate dynamic memory for the integer value stored
 * in the int variable i and attach this new integer value to the growing
 * list of integers the Intlist_T pointer-to-struct points to.  The
 * order of this growing list of integers will be the reverse of the
 * original sequence of integers.
 * ----------
 */

extern T Intlist_push (int i, T list);


/*
 * ----------
 * Intlist_free
 * ==========
 * This function frees all the storage associated with the specified
 * pointer to the Intlist_T pointer-to-struct.
 * ----------
 */

extern void Intlist_free (T *intlist);


/*
 * ----------
 * Intlist_pop
 * ==========
 * This function will take the sequence of integers to whose head the
 * Intlist_T pointer-to-struct list points and extract the last integer
 * that was pushed onto this list of struct Intlist_Ts.  The extracted
 * integer value is stored in the int variable motif_index.  A new
 * Intlist_T pointer-to-struct, which points to the head of a new integers
 * sequence that has been shortened by one integer, will be returned to the
 * user.  To summarize, the last integer to be entered will be the first
 * one to be popped by this function.
 * ----------
 */

extern T Intlist_pop (T list, int *motif_index);


#undef T
#endif
